package week02

import scala.annotation.tailrec

case class Vektor( x : Int, y : Int )  //két adattagú "struct"
/**
 * C megfelelője kb:
 * typedef struct { int x; int y; } Vektor;
 */

trait IVVDV //int vektor vagy double vektor
case class IntVektor( x : Int , y : Int ) extends IVVDV
case class DoubleVektor( xd : Double, yd : Double ) extends IVVDV
/**
 *  C megfelelője kb:
 *  typedef struct { int x; int y; } IntVektor ;
 *  typedef struct { double xd; double yd;} DoubleVektor;
 *  typedef struct {
 *    int type; // 0 -- intvektor    1 -- doublevektor
 *    union{
 *      IntVektor iv;
 *      DoubleVektor dv;
 *    };
 *  }IVVDV;
 */

sealed trait IntLista
case object UresLista extends IntLista //should be a case object
case class NemuresLista( head : Int, tail : IntLista ) extends IntLista

object Eloadas extends App {
  def myPrint( n : Int ) = println( n )

  @tailrec
  def forLoop( from : Int, to : Int, what : Int => Unit ) : Unit =
    if( from > to ) () else { what( from ) ; forLoop( from + 1 , to, what )}

  val n = 10
  forLoop( 1, n, myPrint )            //prints the numbers from 1 to 10
  forLoop( 1, n, { x => println(x) } )        //same but with lambda function
  forLoop( 1, n, { x : Int  => println(x) } ) //same with explicit type

  def add( v1 : Vektor, v2 : Vektor ) = Vektor( v1.x + v2.x , v1.y + v2.y ) //vektorok összege vektor lesz
  val u = Vektor( 0, 1 )
  val v = Vektor( 1, 0 )
  val w = add( u , v )
  println( w )  //prints Vektor(1,1)
  val z = Vektor( -1, 0 )
  println( add(w,z) == u ) //prints true

  val u2 : IVVDV = IntVektor( 8 , 1 )
  val v2 : IVVDV = DoubleVektor( 0.3, 1.2 )
  println( u2.asInstanceOf[IntVektor].x ) //prints 8
  //println( v2.asInstanceOf[IntVektor].x ) //uncomment this to get a ClassCastException run-time

  val nutin = UresLista                 // ()
  val egyelem = NemuresLista( 1, nutin )  // (1)
  val ketelem = NemuresLista( 2, egyelem ) // (2,1)
  val harelem = NemuresLista( 4, ketelem ) // (4,2,1)
  val harelem2 = NemuresLista( 5, ketelem ) // ((5,2,1) , (2,1) part is common in memory

  println( harelem ) //prints NemuresLista(4,NemuresLista(2,NemuresLista(1,UresLista())))
  println( harelem.tail ) // prints NemuresLista(2,NemuresLista(1,UresLista()))
  println(  NemuresLista(4,NemuresLista(2,NemuresLista(1,UresLista))) == harelem ) //true
  println( harelem == harelem2 ) //false

  @tailrec
  def printLista( list : IntLista ) : Unit = list match {
    case UresLista => ()
    case NemuresLista( x, UresLista ) => print( x )
    case NemuresLista( x, t ) => print(x); print(","); printLista(t)
  }
  printLista( harelem ) // prints 4,2,1
  println()

  @tailrec
  def printLista2(list : IntLista ) : Unit = list match {
    case UresLista => ()
    case NemuresLista( x, UresLista ) => print( x )
    case NemuresLista( 7, tail ) => print("HÉT,"); printLista2( tail )
    case NemuresLista( x, tail ) if (x < 0) => print("MÍNUSZ " + -x + ","); printLista2( tail )
    case NemuresLista( x, t ) => print(x); print(","); printLista2(t)
  }
  printLista2( NemuresLista(3,NemuresLista(-2,NemuresLista(
    7,NemuresLista(1,UresLista)))))
  println()
}
